using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Xml;
using System.Xml.Serialization;
using System.Configuration;

namespace DMTServices
{
    static class AppSetting
    {
        private static SettingElement newElement(SettingsProperty pv, ApplicationSettingsBase appSettings)
        {
            SettingElement se = new SettingElement(pv.Name, pv.SerializeAs);
            XmlDocument doc = new XmlDocument();
            XmlElement xmlValue = doc.CreateElement("value");

            string s = string.Empty;
            if (appSettings[pv.Name] == null)
                s = pv.DefaultValue.ToString();
            else
            {
                if (pv.SerializeAs == SettingsSerializeAs.Xml)
                {
                    XmlSerializer xs = new XmlSerializer(appSettings[pv.Name].GetType());
                    System.IO.TextWriter myxml = new System.IO.StringWriter();
                    xs.Serialize(myxml, appSettings[pv.Name]);
                    s = myxml.ToString();
                }
                else
                {
                    TypeConverter converter = TypeDescriptor.GetConverter(appSettings[pv.Name].GetType());
                    s = converter.ConvertToString(appSettings[pv.Name]);
                }
            }

            if (pv.SerializeAs == SettingsSerializeAs.Xml)
            {
                // there seems to be a 'header' that we needed to remove to get this to work?
                //  ie.   <?xml version="1.0" encoding="utf-16"?>
                int i = s.IndexOf("?>");
                if (i > 0)
                    s = s.Substring(i + 2);
                xmlValue.InnerXml = s;
            }
            else
                xmlValue.InnerText = s;

            SettingValueElement valueElem = new SettingValueElement();
            valueElem.ValueXml = xmlValue;
            se.Value = valueElem;
            return se;
        }

        public static void SaveSettings(ApplicationSettingsBase appSettings)
        {
            ClientSettingsSection newSection;
            try
            {
                // I decided to use the config file name to avoid issues when running in debug where it used the .vshost. one
                string name = System.Reflection.Assembly.GetExecutingAssembly().Location;
                System.Configuration.Configuration config = ConfigurationManager.OpenExeConfiguration(name);
                ConfigurationSectionGroup group = config.SectionGroups["applicationSettings"];

                if (group == null)
                {
                    // if there is no applicationSettings section at all, then create everything...
                    config.SectionGroups.Add("applicationSettings", new ApplicationSettingsGroup());
                    group = config.SectionGroups["applicationSettings"];

                    newSection = new ClientSettingsSection();
                    foreach (SettingsProperty pv in appSettings.Properties)
                        // Leave the User settings alone, we are only interested in Application scoped ones...
                        if (!pv.Attributes.ContainsValue(new UserScopedSettingAttribute()))
                            newSection.Settings.Add(newElement(pv, appSettings));

                    // there is always a section under applicationSettings for the 'default' stuff
                    group.Sections.Add(appSettings.Context["GroupName"].ToString(), newSection);
                }
                else
                {
                    foreach (ConfigurationSection section in group.Sections)
                    {
                        if ((section as ClientSettingsSection) == null)
                            continue;

                        newSection = new ClientSettingsSection();
                        foreach (SettingsProperty pv in appSettings.Properties)
                            // Leave the User settings alone, we are only interested in Application scoped ones...
                            if (!pv.Attributes.ContainsValue(new UserScopedSettingAttribute()))
                                newSection.Settings.Add(newElement(pv, appSettings));

                        group.Sections.Remove(section.SectionInformation.Name);
                        group.Sections.Add(section.SectionInformation.Name, newSection);
                    }
                }
                config.Save(ConfigurationSaveMode.Full);
            }
            catch // (Exception e) 
            {
                //System.Windows.Forms.MessageBox.Show(e.Message);
            }
        }
    }
}
