﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DMTServices;
using System.IO;


namespace CompuCS440Simulator
{
    public partial class frmMain : Form
    {
        int rxStep = 0;
        byte[] txBuf = new byte[88];

        public frmMain()
        {           
            InitializeComponent();                   
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            timer.Interval = 5000;                 
        }

        private void txtEvt_Num_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != 8)
                e.Handled = true;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            try
            {
                rxStep = 0;
                FillBuf();
                serialPort.PortName = cboPort.Text;
                serialPort.Open();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            if (serialPort.IsOpen)
            {
                timer.Enabled = true;

                btnStop.Enabled = true;
                btnRefresh.Enabled = true;
                btnStart.Enabled = false;
            }
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (serialPort.IsOpen)
            {
                try
                {
                    serialPort.Close();
                }
                catch { }
            }
            btnStart.Enabled = true;
            btnStop.Enabled = false;
            btnRefresh.Enabled = false;

            timer.Enabled = false; 
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            timer.Enabled = false;
            FillBuf();
            Properties.Settings.Default.Save();
            timer.Enabled = true; 
        }

        private void chkBlasting_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void chkExtended_CheckedChanged(object sender, EventArgs e)
        {
            pnlExtended.Enabled = chkExtended.Checked;
            if (chkExtended.Checked)
                txBuf = new byte[128];
            else
                txBuf = new byte[88];
            FillBuf();
        }

        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            string s = serialPort.ReadExisting();
            foreach (char c in s)
            {
                switch (rxStep)
                {
                    case 0:
                        if (c == '@')
                            rxStep++;
                        break;
                    case 1:
                        if (c == '1')
                        {
                            CalcChecksum();
                            serialPort.Write(txBuf, 0, txBuf.Length);
                            rxStep = 0;
                        }
                        else
                            if (c != '@')
                                rxStep = 0;
                        break;
                }
            }
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            SimulateData();
            FillBuf();
            Properties.Settings.Default.Save();
        }


        private int atoi(string s)
        {
            int Result = 0;
            try
            {
                Result = Convert.ToInt32(s);
            }
            catch {}
            
            return Result;
        }
        
        private void StringToBuf16(string s, int pos)
        {
            IntToBuf16(atoi(s), pos);
        }

        private void IntToBuf16(int i, int pos)
        {
            txBuf[pos++] = (byte)(i & 0xff);
            txBuf[pos] = (byte)((i >> 8) & 0xff);
        }

        private void StringToBuf8(string s, int pos)
        {
            IntToBuf8(atoi(s), pos);
        }

        private void IntToBuf8(int i, int pos)
        {
            txBuf[pos] = (byte)(i & 0xff);
        }

        private void FillBuf()
        {
            int i;
/*            int evtNum;
            try
            {
                evtNum = Convert.ToInt32(txtEvt_Num.Text);
            }
            catch
            {
                evtNum = 0;
            }
            evtNum++;
            txtEvt_Num.Text = evtNum.ToString(); */
            IntToBuf16(atoi(txtEvt_Num.Text), 0);
            
            i = chkBlasting.Checked ? 1 : 0;
            i |= chkUnloading.Checked ? 2 : 0;
            i |= chkPause.Checked ? 4 : 0;
            i |= atoi(txtPavementTemp.Text) << 8;
            IntToBuf16(i, 2);
            IntToBuf16((atoi(txtAirTemp.Text) << 8) | atoi(txtMode.Text) | (chkLiquidOnly.Checked ? 16 : 0), 4);
            StringToBuf16(txtSpinRPM.Text, 6);
            StringToBuf16(txtRealAppRate.Text, 8);
            StringToBuf16(txtRealLiquidRate.Text, 10);
            StringToBuf16(txtSpinnerSetpoint.Text, 12);
            StringToBuf16(txtActualAppRate.Text, 14);
            StringToBuf16(txtActualPrewet.Text, 16);
            StringToBuf16(txtLogAppRate.Text, 18);
            IntToBuf16(0, 20);  // hi word time
            IntToBuf16(0, 22);  // lo word time
            StringToBuf16(txtLogTotalTime.Text, 24);
            StringToBuf16(txtLogBlastTime.Text, 26);
            StringToBuf16(txtLogPauseTime.Text, 28);
            StringToBuf16(txtLogTotalDistance.Text, 30);
            StringToBuf16(txtLogBlastDistance.Text, 32);
            StringToBuf16(txtLogPauseDistance.Text, 34);
            StringToBuf16(txtLogTotalSolid.Text, 36);
            StringToBuf16(txtLogBlastSolid.Text, 38);
            StringToBuf16(txtLogTotalLiquid.Text, 40);
            StringToBuf16(txtLogLiquidSet.Text, 42);
            StringToBuf16(txtLogSpinSet.Text, 44);
            StringToBuf8(txtLogCelsius.Text, 46);
            StringToBuf8(txtLogIndex.Text, 47);
            StringToBuf8(txtLogError.Text, 48);
            StringToBuf8(txtLogBlastNum.Text, 49);
            StringToBuf8(txtLogGate.Text, 50);
            StringToBuf8(txtLogCtrlMode.Text, 51);
            StringToBuf16(txtLiquidSet.Text, 52);
            StringToBuf16(txtTripSpreadDistance.Text, 54);
            StringToBuf16(txtTripSpreadTime.Text, 56);
            StringToBuf16(txtTripSpreadQuantity.Text, 58);
            StringToBuf16(txtTripSpreadVolume.Text, 60);
            StringToBuf16(txtLogTotalVDeice.Text, 62);
            StringToBuf8(txtCurrentGate.Text, 64);
            IntToBuf8((atoi(txtCurMatSolid.Text) & 0x3) | ((atoi(txtCurMatLiquid.Text) & 0x3) << 2), 65);

            string tid = txtTruckId.Text.PadRight(10);
            Array.Copy(Encoding.ASCII.GetBytes(tid), 0, txBuf, 66, 10);
            txBuf[76] = 32;     // space, not sure why?
            StringToBuf8(txtErrCode.Text, 77);

            string did = txtDriverId.Text.PadRight(8);
            Array.Copy(Encoding.ASCII.GetBytes(did), 0, txBuf, 78, 8);

            if (chkExtended.Checked)
            {
                StringToBuf16(txtTripSprdDist.Text, 86);
                StringToBuf16(txtTripSprdTm.Text, 88);
                StringToBuf16(txtTripTotalLiq.Text, 90);
                StringToBuf16(txtLiqSet.Text, 96);
                StringToBuf16(txtLiqAct.Text, 98);
                StringToBuf16(txtJoyStick.Text, 100);
            }

#if TEST_MODE           
            AppendAllBytes(txBuf);
#endif

        }
       
        private void CalcChecksum()
        {
            int cks = 0;
            for(int i = 0; i < txBuf.Length - 2; i += 2)
                cks += txBuf[i] | (txBuf[i + 1] << 8);
            txBuf[txBuf.Length - 2] = (byte)(cks & 0xff);
            txBuf[txBuf.Length - 1] = (byte)(cks >> 8);
        }
                
        private void SimulateData()
        {                                     
            double dspeed = Convert.ToDouble(numUDSpeed.Value);
            //In one minute, you can figure out how many kms the vehicle travelled
            double minDist = dspeed / 60;  // 50/60 = 0.8333 km.
            double secDist  = dspeed / 3600;
            // timer interval = 5 sec ( 5000);
            double tSecInterval = timer.Interval / 1000;  // to get the number of seconds in the interval, divide this number by 1,000.

            //What we need is to have the 
            //“Log Total Solid” and possibly has an option the 
            //“Log Total Liquid” fields 
            //auto-increment based on the “Log App Rate” and “Log Liquid Set” values.   
            double dLogAppRate = myConvert.StrToFloatDef(txtLogAppRate.Text, 1);
            double dLogLiquidSet = myConvert.StrToFloatDef(txtLogLiquidSet.Text, 1);

            //If you’re running at say 160 lbs/km,
            // then in one minute the Log Total Solid should increment 160 * 0.83333. 
            //You can take that amount and say divide it by 12 giving you how much to increment the value every 5 seconds

            //double incLogTotalSolid = dLogAppRate * minDistance / 12;   // 60 sec / 5 sec = 12; 
            //double incLogTotalLiquid = dLogLiquidSet * minDistance / 12; 

            // because the timer displays data every 5 seconds : 
            double incLogTotalSolid = dLogAppRate * secDist * tSecInterval;
            double incLogTotalLiquid = dLogLiquidSet * secDist * tSecInterval;

            txtLogTotalSolid.Text = string.Format("{0:0.00}",
                                    myConvert.StrToFloatDef(txtLogTotalSolid.Text, 0) + incLogTotalSolid);
            
            txtTripSpreadQuantity.Text = string.Format("{0:0.00}",
                                    myConvert.StrToFloatDef(txtTripSpreadQuantity.Text, 0) + incLogTotalSolid);

            txtLogTotalLiquid.Text = string.Format("{0:0.00}",
                                     myConvert.StrToFloatDef(txtLogTotalLiquid.Text, 0) + incLogTotalLiquid);

            txtLogTotalDistance.Text = string.Format("{0:0.00}",
                                       myConvert.StrToFloatDef(txtLogTotalDistance.Text, 0) + secDist * tSecInterval);

            txtTripSpreadDistance.Text = string.Format("{0:0.00}",
                                       myConvert.StrToFloatDef(txtTripSpreadDistance.Text, 0) + secDist * tSecInterval);


        }
      

#if TEST_MODE
        private void AppendAllBytes(byte[] bytes) 
        {
            System.IO.File.AppendAllText(@"c:\temp\simulator\here.txt", DateTime.Now.ToString());
            //argument-checking here.
            using (var stream = new FileStream(@"c:\temp\simulator\here.txt", FileMode.Append))
            {
                stream.Write(bytes, 0, bytes.Length);
            }
        }

        private void WriteByteArrayToFile(byte[] fileBytes)
        {
            System.IO.File.WriteAllText(@"c:\temp\simulator\here.txt", DateTime.Now.ToString());
            using (Stream file = File.OpenWrite(@"c:\temp\simulator\here.txt"))
            {
                file.Write(fileBytes, 0, fileBytes.Length);
            }
        }

#endif
    
        /*     You can actually do the math based on the “Log App Rate” value.  
         *     This is in lbs/km, therefore let’s say Allan drives the vehicle at 50 km/hr.  
         *     In one minute, you can figure out how many kms the vehicle travelled (50 / 60 = 0.8333 km).  
         *     If you’re running at say 160 lbs/km, 
         *     then in one minute the Log Total Solid should increment 160 * 0.83333.    
         *     You can take that amount and say divide it by 12 giving you how much to increment the value every 5 seconds.  
         *     So, if you set a timer a 5 second intervals, you know how much to increment the value.  
         *     The liquid is similar, the rate is in litres / km I believe.  
         *     Maybe the user could enter the vehicle speed and have a spinner that would allow them to change the speed up and down as the drive around, 
         *     this would automatically change the math and how much the total material increments by.   
         *     I’m not sure, but the 
         *     Log Total Distance may also have to increment to make the spreader reports correct.
         */      

    }
}


#if DELETE
  /*        
        //Read file to byte array
        FileStream stream = File.OpenRead(@"c:\path\to\your\file\here.txt");
        byte[] fileBytes= new byte[stream.Length]; 
        stream.Read(fileBytes, 0, fileBytes.Length);
        stream.Close();
        //Begins the process of writing the byte array back to a file
        using (Stream file = File.OpenWrite(@"c:\path\to\your\file\here.txt"))
        {
           file.Write(fileBytes, 0, fileBytes.Length);
        }*/
#endif