﻿using System;
//using System.Data;
//using System.Data.Common;
using System.Text;
//#if !NET20
//using System.Linq;
//#endif
//using System.Configuration;
//using System.Collections;
//using System.Collections.Generic;
using System.Globalization;


namespace DMTServices
{
    /// <summary>
    /// Class for data conversion
    /// WITHOUT exceptions and as fast as possible...
    ///  (I put it here as these are called extensively from the data connection classes above)
    /// </summary>
    public static class myConvert
    {
        public static string ToString(object o)
        {
            return ToString(o, string.Empty);
        }

        public static string ToString(object o, string def)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                    return Convert.ToString(o);
            }
            catch
            {
                return def;
            }
        }

        public static bool ToBoolean(object o)
        {
            return ToBoolean(o, false);
        }

        public static bool ToBoolean(object o, bool def)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                {
                    if (o is string || o is char)
                    {
                        bool b;
                        if (!bool.TryParse((o is char) ? ((char)o).ToString() : (string)o, out b))
                            b = def;
                        return b;
                    }
                    else
                        if (o is DateTime)
                            return def;
                        else
                            return Convert.ToBoolean(o);
                }
            }
            catch
            {
                return def;
            }
        }

        public static double ToDouble(object o)
        {
            return ToDouble(o, 0.0);
        }

        public static double ToDouble(object o, double def)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                {
                    if (o is string || o is char)
                    {
                        double d;
                        if (!double.TryParse((o is char) ? ((char)o).ToString() : (string)o, out d))
                            d = def;
                        return d;
                    }
                    else
                        if (o is DateTime)
                            return ((DateTime)o).Ticks / 10000;     // milliseconds since DateTime.MinValue
                        else
                            return Convert.ToDouble(o);
                }
            }
            catch
            {
                return def;
            }
        }

        public static int ToInt(object o)
        {
            return ToInt32(o, 0);
        }

        public static int ToInt(object o, int def)
        {
            return ToInt32(o, def);
        }

        public static int ToInt32(object o)
        {
            return ToInt32(o, 0);
        }

        public static int ToInt32(object o, int def)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                {
                    // the main time delay is when an Exception occurs so,
                    // by handling strings this way, non numeric data will not cause one...
                    if (o is string || o is char)
                    {
                        int i;
                        if (!int.TryParse((o is char) ? ((char)o).ToString() : (string)o, out i))
                            i = def;
                        return i;
                    }
                    else
                        if (o is bool)
                            return ((bool)o) ? 1 : 0;
                        else
                            // again, anything to avoid an exception,
                            // if someone asks for the int value of a DateTime
                            // what do they expect to get?
                            if (o is DateTime)
                                return (int)(((DateTime)o).Ticks / 600000000);      // minutes since DateTime.MinValue
                            else
                            {
                                // try to stop numbers too big or too small from causing Exceptions...
                                if (o is double)
                                    if ((double)o >= Int32.MaxValue)
                                        return Int32.MaxValue;
                                    else
                                        if ((double)o <= Int32.MinValue)
                                            return Int32.MinValue;
                                return Convert.ToInt32(o);
                            }
                }
            }
            catch
            {
                return def;
            }
        }

        // these 2 funcs are not really doing what you might think, but they will give a value
        // based on the first 3 chars of the string...
        public static Int64 AsciiToInt(object o)
        {
            return AsciiToInt(o, 0);
        }

        public static Int64 AsciiToInt(object o, int def)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                {
                    if (o is string || o is char)
                    {
                        string val = (o is char) ? ((char)o).ToString() : (string)o;
                        Int64 i = 0;
                        int j = 0;
                        foreach (char c in val)
                        {
                            i *= 256;
                            i += c;
                            if (++j > 2)
                                break;
                        }
                        for (; j < 3; j++)
                            i *= 256;

                        return i;
                    }
                    else
                        return def;
                }
            }
            catch
            {
                return def;
            }
        }

        public static Int64 ToInt64(object o)
        {
            return ToInt64(o, 0);
        }

        public static Int64 ToInt64(object o, Int64 def)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                {
                    // the main time delay is when an Exception occurs so,
                    // by handling strings this way, non numeric data will not cause one...
                    if (o is string || o is char)
                    {
                        Int64 i;
                        if (!Int64.TryParse((o is char) ? ((char)o).ToString() : (string)o, out i))
                            i = def;
                        return i;
                    }
                    else
                        if (o is bool)
                            return ((bool)o) ? 1 : 0;
                        else
                            // again, anything to avoid an exception,
                            // if someone asks for the int value of a DateTime
                            // what do they expect to get?
                            if (o is DateTime)
                                return (Int64)(((DateTime)o).Ticks / 600000000);      // minutes since DateTime.MinValue
                            else
                            {
                                // try to stop numbers too big or too small from causing Exceptions...
                                if (o is double)
                                    if ((double)o >= Int64.MaxValue)
                                        return Int64.MaxValue;
                                    else
                                        if ((double)o <= Int64.MinValue)
                                            return Int64.MinValue;
                                return Convert.ToInt64(o);
                            }
                }
            }
            catch
            {
                return def;
            }
        }

        public static DateTime ToDateTime(object o)
        {
            return ToDateTime(o, DateTime.MinValue, string.Empty);
        }

        public static DateTime ToDateTime(object o, DateTime def)
        {
            return ToDateTime(o, def, string.Empty);
        }

        // new CultureInfo("en-US").DateTimeFormat
        public static DateTime ToDateTime(object o, DateTime def, string fmt)
        {
            try
            {
                if (o == null || o is System.DBNull)
                    return def;
                else
                {
                    if (o is string)
                    {
                        DateTime dt;
                        if (!string.IsNullOrEmpty(fmt))
                        {
                            if (!DateTime.TryParseExact((string)o, fmt, null, DateTimeStyles.AllowWhiteSpaces | DateTimeStyles.NoCurrentDateDefault, out dt))
                                dt = def;
                        }
                        else
                            if (!DateTime.TryParse((string)o, out dt))
                                dt = def;
                        return dt;
                    }
                    else
                        if (o is double)
                            return DateTime.MinValue.AddMilliseconds((double)o);
                        else
                            if (o is int)
                                return DateTime.MinValue.AddMinutes((double)((int)o));
                            else
                                if (o is DateTime)
                                    return (DateTime)o;
                                else
                                    return def;
                }
            }
            catch
            {
                return def;
            }
        }

        public static DateTime ToTime(string time, DateTime date)
        {
            string fmt = "HHmm";
            if (time.Contains(":"))
            {
                int c = time.Length;
                time = time.Replace(":", "");
                if ((c - time.Length) > 1)  // if there was more than 1 colon, ass-u-me seconds included...
                    fmt += "ss";
            }

            int l = fmt.Length;
            if (time.Length > l)
                time = time.Substring(time.Length - l);
            else
                time = time.PadLeft(fmt.Length, '0');

            DateTime rc = ToDateTime(time, DateTime.MinValue, fmt);
            //if (rc != DateTime.MinValue)      if time passed in is "0", rc WILL BE MinValue, but we should still add it to date.Date
            rc = date.Date.Add(rc.TimeOfDay);
            return rc;
        }

        public static byte[] ToByteArray(object o)
        {
            try
            {
                if (o != null && !(o is DBNull))
                    return (byte[])o;
                else
                    return null;
            }
            catch
            {
                return null;
            }
        }

        public static int StrToIntDef(string s, int def)
        {
            return ToInt32(s, def);
        }

        public static double StrToFloatDef(string s, double def)
        {
            return ToDouble(s, def);
        }

        public static DateTime StrToDate(string s)
        {
            return ToDateTime(s, DateTime.MinValue, string.Empty);
        }

        public static DateTime StrToDate(string s, string fmt)
        {
            return ToDateTime(s, DateTime.MinValue, fmt);
        }

        public static string ByteArrayToString(byte[] abData)
        {
            StringBuilder psBuilder = new StringBuilder(abData.Length * 2);

            for (int i = 0; i < abData.Length; i++)
                psBuilder.AppendFormat("{0:X2}", abData[i]);

            return psBuilder.ToString();
        }

    }
}